
// Copyright (c) 2010, Ryan Linneman. All rights reserved.
// This file is under the BSD License, refer to license.txt for details

/*
 * Converted 23/12/2009 to use the ffNBAScoreboard namespace
 * http://blogger.ziesemer.com/2007/10/respecting-javascript-global-namespace.html
 *
 */

if (!ffNBAScoreboard) var ffNBAScoreboard = {};

ffNBAScoreboard = {

// Specify constants for configuring the toolbar button drop down menu ------ //
LIVE_SCORES_MENUITEM: 4,
DEFAULT_MENUITEM_TOTAL: 13,

// Specify the branch of the preference tree to access ---------------------- //
nbaPrefBranch: Components.classes["@mozilla.org/preferences-service;1"]
                         .getService(Components.interfaces.nsIPrefService)
                         .getBranch("extensions.nbaScoreboard."),


// -------------------------------------------------------------------------- //
// Functions called from nbascoreboard.xul                                    //
// -------------------------------------------------------------------------- //

/*
 * OpenScoreboard
 *
 * aDaysAgo: 0: Today, 1: Yesterday, 2: Two Days Ago, etc
 * Additional argument may be supplied by XUL menu oncommand
 *
 */

OpenScoreboard: function (aDaysAgo) {

  // Retrieve the scoreboard preference value
  var scoreboardType = this.nbaPrefBranch.getCharPref("scoreboard");

  // Overwrite if an additional argument is supplied (XUL menu oncommand)
  if (arguments.length > 1) {
    scoreboardType = arguments[1];
  }

  // Get the date of Today's Scoreboard
  var scoreboardDate = this.GetTodaysDate(scoreboardType);

  // Find previous scoreboard dates
  while (aDaysAgo > 0) {
    scoreboardDate = this.GetPreviousDate(scoreboardDate, scoreboardType);
    aDaysAgo--;
  }

  // Declare scoreboardURL variable
  var scoreboardURL = "";

  // Construct the scoreboard URL
  switch (scoreboardType) {

  case "nba":
    scoreboardURL = "http://www.nba.com/gameline/" + scoreboardDate + "/";
    break;

  case "espn":
    scoreboardURL = "http://scores.espn.go.com/nba/scoreboard?date=" + scoreboardDate;
    break;

  case "si":
    scoreboardURL = "http://sportsillustrated.cnn.com/basketball/nba/scoreboards/" + scoreboardDate + "/index.html";
    break;

  case "cbs":
    scoreboardURL = "http://www.cbssports.com/nba/scoreboard/" + scoreboardDate;
    break;

  case "yahoo":
    scoreboardURL = "http://sports.yahoo.com/nba/scoreboard?d=" + scoreboardDate;
    break;

  default:
    scoreboardURL = "http://www.nba.com/gameline/" + scoreboardDate + "/";

  }

  // Open the NBA scoreboard URL
  this.OpenURL(scoreboardURL, true);

},


/*
 * ShowLeaders
 *
 */

ShowLeaders: function () {

  // Load the Leaderboard URL and set the features required
  var leadersURL = "chrome://nbascoreboard/content/nba-leaderboard.xul";
  var features = "chrome,titlebar,toolbar,centerscreen";

  // Open the Leaderboard Window
  window.openDialog(leadersURL, "", features);

},


/*
 * ShowStandings
 *
 */

ShowStandings: function () {

  // Retrieve the standings preference value and complete the URL
  standingsType = this.nbaPrefBranch.getCharPref("standings");

  // Construct the NBA standings URL
  var standingsURL = "http://sportsillustrated.cnn.com/basketball/nba/standings/";

  // Add the conference based on the preference
  if (standingsType == "Cnf") {
    standingsURL += "conference/index.html";
  }

  // Open the NBA standings URL
  this.OpenURL(standingsURL, false);

},


/*
 * ShowOptions
 *
 */

ShowOptions: function () {

  // Get the root branch
  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                        .getService(Components.interfaces.nsIPrefBranch);

  // Get the instantApply preference
  var instantApply = prefs.getBoolPref("browser.preferences.instantApply", false);

  // Load the Options dialog URL and set the features required
  var optionsURL = "chrome://nbascoreboard/content/nba-options.xul";
  var features = "chrome,titlebar,toolbar,centerscreen" + (instantApply ? ",dialog=no" : ",modal");

  // Open the Options Window
  openDialog(optionsURL, "", features);

},


/*
 * OpenGameItem
 *
 * aLinkId: A link id with the format 'nba-xxx-YYYY/MM/DD/GAMEID'
 * - xxx is either 'rcp', 'box' or 'pre'
 *
 */

OpenGameItem: function (aLinkId) {

  // Split the link ID into linkInfo[linkType, linkURL]
  var linkInfo = aLinkId.split("-");

  // Construct the URL base
  var linkUrl = "http://sportsillustrated.cnn.com/basketball/nba/gameflash/";

  // Append the YYYY/MM/DD/GAMEID
  linkUrl += linkInfo[2];

  // Complete the URL
  if (linkInfo[1] == "rcp") {
    linkUrl += "_recap.html";
  }
  else if (linkInfo[1] == "box") {
    linkUrl += "_boxscore.html";
  }
  else if (linkInfo[1] == "pre") {
    linkUrl += "_preview.html";
  }

  // Open the link URL
  this.OpenURL(linkUrl, false);

},


/*
 * OpenURL
 *
 * aURL: The URL information
 * aFocus: If opening in a new tab, give new tab the focus
 *
 */

OpenURL: function (aURL, aFocus) {

  // CODE TAKEN FROM https://developer.mozilla.org/En/Code_snippets/Tabbed_browser

  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
  var browserEnumerator = wm.getEnumerator("navigator:browser");

  // Check each browser instance for our URL
  var found = false;

  while (!found && browserEnumerator.hasMoreElements()) {
    var browserWin = browserEnumerator.getNext();
    var tabbrowser = browserWin.gBrowser;

    // Check each tab of this browser instance
    var numTabs = tabbrowser.browsers.length;
    for (var index = 0; index < numTabs; index++) {
      var currentBrowser = tabbrowser.getBrowserAtIndex(index);
      if (aURL == currentBrowser.currentURI.spec) {

        // The URL is already opened. Select this tab.
        tabbrowser.selectedTab = tabbrowser.tabContainer.childNodes[index];

        // Focus *this* browser-window
        browserWin.focus();

        found = true;
        break;
      }
    }
  }

  if (!found) {

    // Open the URL in the same tab if the current tab is empty
    if (gBrowser.currentURI.spec == "about:blank") {
      gBrowser.loadURI(aURL);
    }

    // Open the URL in a new tab
    else { 

      // Give focus to that tab
      if (aFocus) {
        gBrowser.selectedTab = gBrowser.addTab(aURL);
      }

      // Do not give focus to that tab
      else {
        gBrowser.addTab(aURL);
      }

    }

  }

},


/*
 * LoadTodaysGames
 *
 */

LoadTodaysGames: function () {

  // Get a reference to the ffNBAScoreboard object
  let that = this;

  // Get the boxscore menu element to populate
  var boxscoreMenu = document.getElementById("nbascoreboard-menu");

  // Get the recap menupopup element to populate 
  var recapMenu = document.getElementById("nbascoreboard-TR-popup");

  // Purge the menu structure of dynamic modifications
  this.PurgeMenu();

  // Configure the main popup menu's preferences
  this.ConfigureMenu();

  // Add a 'loading' menu item to the boxscore menu
  var loadingItem = document.createElement("menuitem");
  loadingItem.setAttribute("id", "loadingItem");
  loadingItem.setAttribute("label", "Connecting to data source...");
  loadingItem.setAttribute("disabled", true);
  boxscoreMenu.insertBefore(loadingItem, boxscoreMenu.childNodes.item(this.LIVE_SCORES_MENUITEM));

  // Change to the wait cursor
  window.setCursor("progress");

  // Get the date of Today's Scoreboard
  var scoreboardDate = this.GetTodaysDate("si");

  // Construct the NBA scoreboard data URL
  var scoreboardURL = "http://sportsillustrated.cnn.com/basketball/nba/scoreboards/" + scoreboardDate + "/scoreboard.dat";

  // XMLHttpRequest 'progress' event function
  function updateProgress(evt) {

    if (evt.lengthComputable) {
      var percentComplete = parseInt((evt.loaded / evt.total) * 100, 10);
      document.getElementById("loadingItem").setAttribute("label", "Loading (" + percentComplete + "%)");
    }

  }

  // XMLHttpRequest 'load' event function
  function transferComplete(evt) {

    // Remove the 'loading' menu item from the menu
    boxscoreMenu.removeChild(boxscoreMenu.childNodes.item(that.LIVE_SCORES_MENUITEM));

    // Check that the HTTP status is OK
    if (req.status == 200) {

      // Store the ResponseText
      var nbaScoreData = req.responseText;

      // Extract an array of games from the ResponseText
      var nbaGames = nbaScoreData.split(/\n/);

      // Sort the game array
      nbaGames = that.SortGameArray(nbaGames);

      // Step through the NBA games for the day
      for (var i = 1; i < nbaGames.length - 1; i++) {

        // Split the game data
        var gameData = nbaGames[i].split("|");
        var gameLinks = gameData[10].split(";");
        var gameState = gameData[8].split(";");
        var htm = gameData[11].split(";");
        var vtm = gameData[12].split(";");

        // Extract the status of the current game
        var gameStatus = gameData[5];

        // Extract the status description of the current game
        var gameStatText = that.GetGameStatText(gameState[1], gameState[2], gameState[3], gameData[7]);

        // Extract the code for the current game
        var gameCode = gameData[0];

        // Extract the home team info
        var htmName = that.GetTeam(htm[3], htm[2], htm[4]);
        var htmScore = htm[6];

        // Extract the visitor team info
        var vtmName = that.GetTeam(vtm[3], vtm[2], vtm[4]);
        var vtmScore = vtm[6];

        // Rate the game for determining menu item icon
        var gameRating = that.GetGameRating(gameState[2], htmScore, vtmScore);

        // Create a new menu item to add to the boxscore menu
        var boxscoreItem = document.createElement("menuitem");
        var itemId = "";
        var itemLabel = "";

        // Create a game preview menu item
        if (gameStatus == "" || gameStatus == "PRE" || gameStatus == "SCHE") {

          // Construct the boxscoreItem's ID
          itemId = "nba-pre-" + scoreboardDate + "/" + gameCode;

          // Construct the boxscoreItem's label
          itemLabel = htmName + " v " + vtmName + ", " + gameStatText;

          // Style the menu item (colour and icon)
          boxscoreItem.setAttribute("class", "menuitem-iconic nbascoreboard-preview nbascoreboard-rating-" + gameRating);

          // Set the tooltip text
          boxscoreItem.setAttribute("tooltiptext", "Opens the NBA game preview");

        }

        // Create a game complete menu item
        else if (gameStatus == "SUSP" || gameStatus == "CANC" || gameStatus =="FINA") {

          // Construct the boxscoreItem's ID
          itemId = "nba-box-" + scoreboardDate + "/" + gameCode;

          // Construct the boxscoreItem's label
          itemLabel = htmName + " " + htmScore + " | " + vtmName + " " + vtmScore;

          // Append overtime alert if applicable
          if (gameState[2] > 4) {
            itemLabel += " (OT)";
          }

          // Style the menu item (colour and icon)
          boxscoreItem.setAttribute("class", "menuitem-iconic nbascoreboard-rating-" + gameRating);

          // Set the tooltip text
          boxscoreItem.setAttribute("tooltiptext", "Opens the NBA game boxscore");

          // Check for a recap to add to "nbascoreboard-TR-menu"
          if (gameLinks[2] == "T") {

            // Create a new menu item to add to the popup menu
            var recapItem = document.createElement("menuitem");

            // Set the recap item's attributes
            recapItem.setAttribute("id", "nba-rcp-" + scoreboardDate + "/" + gameCode);
            recapItem.setAttribute("label", itemLabel);
            recapItem.setAttribute("class", "menuitem-iconic nbascoreboard-rating-" + gameRating);
            recapItem.setAttribute("oncommand", "ffNBAScoreboard.OpenGameItem(id); event.stopPropagation();");
            recapItem.setAttribute("tooltiptext", "Opens the NBA game recap");

            // Add the recapItem to the recapMenu
            recapMenu.insertBefore(recapItem, recapMenu.firstChild);

            // Enable the recap menupopup
            document.getElementById("nbascoreboard-TR-menu").setAttribute("hidden", false);

          }

        }

        // Create a game in progress menu item (PROG, END-, DELA)
        else {

          // Construct the boxscoreItem's ID
          itemId = "nba-box-" + scoreboardDate + "/" + gameCode;

          // Construct the boxscoreItem's label
          itemLabel = htmName + " " + htmScore + " | " + vtmName + " " + vtmScore + " (" + gameStatText + ")";

          // Style the menu item (colour and icon)
          boxscoreItem.setAttribute("class", "menuitem-iconic nbascoreboard-progress nbascoreboard-rating-" + gameRating);

          // Set the tooltip text
          boxscoreItem.setAttribute("tooltiptext", "Opens the NBA game boxscore");

        }

        // Highlight favourite team
        if (htm[3] == that.nbaPrefBranch.getCharPref("favTeam") || vtm[3] == that.nbaPrefBranch.getCharPref("favTeam")) {

          // Apply highlight
          var classAttribute = boxscoreItem.getAttribute("class");
          classAttribute += " nbascoreboard-favteam";
          boxscoreItem.setAttribute("class", classAttribute);

        }

        // Set the new menu item's attributes
        boxscoreItem.setAttribute("id", itemId);
        boxscoreItem.setAttribute("label", itemLabel);
        boxscoreItem.setAttribute("oncommand", "ffNBAScoreboard.OpenGameItem(id); event.stopPropagation();");

        // Add the menu item to the popup menu
        boxscoreMenu.insertBefore(boxscoreItem, boxscoreMenu.childNodes.item(that.LIVE_SCORES_MENUITEM));

      }

      // Insert a blank item if there are no games
      if (boxscoreMenu.childNodes.length == that.DEFAULT_MENUITEM_TOTAL) {

        // Create a new menu item to add to the popup menu
        var blankItemNoScores = that.CreateBlankMenuItem("blankItem", "There are no games today", "No game information");

        // Add the item to the menu
        boxscoreMenu.insertBefore(blankItemNoScores, boxscoreMenu.childNodes.item(that.LIVE_SCORES_MENUITEM));

      }

    // HTTP Request is not OK - boxscores not available
    } else {

      // Create a new menu item to add to the popup menu
      var blankItemNoData = that.CreateBlankMenuItem("blankItem", "Error connecting to data source", "HTTP Error: " + req.status);

      // Add the item to the menu
      boxscoreMenu.insertBefore(blankItemNoData, boxscoreMenu.childNodes.item(that.LIVE_SCORES_MENUITEM));

    }

    // Change to the default cursor
    window.setCursor("auto");

  }

  // Initialise an XMLHttpRequest object for loading XML
  var req = new XMLHttpRequest();

  // Add event listeners for XMLHttpRequest
  req.addEventListener("progress", updateProgress, false);
  req.addEventListener("load", transferComplete, false);

  // Load the XML data source of the required scoreboard
  req.open("GET", scoreboardURL, true);

  // Initiate the XMLHttpRequest
  req.send(null);

},


/*
 * LoadYesterdaysGames
 *
 */

LoadYesterdaysGames: function () {

  // Get a reference to the ffNBAScoreboard object
  let that = this;

  // Get the boxscore menupopup element to populate
  var boxscoreMenu = document.getElementById("nbascoreboard-YB-popup");

  // Get the recap menupopup element to populate 
  var recapMenu = document.getElementById("nbascoreboard-YR-popup");

  // XMLHttpRequest 'progress' event function
  function updateProgress(evt) {

    if (evt.lengthComputable) {
      var percentComplete = parseInt((evt.loaded / evt.total) * 100, 10);
      document.getElementById("boxLoadingItem").setAttribute("label", "Loading (" + percentComplete + "%)");
      document.getElementById("recapLoadingItem").setAttribute("label", "Loading (" + percentComplete + "%)");
    }

  }

  // XMLHttpRequest 'load' event function
  function transferComplete(evt) {

    // Remove the 'loading' menu item from the popup menus
    boxscoreMenu.removeChild(boxscoreMenu.childNodes.item(0));
    recapMenu.removeChild(recapMenu.childNodes.item(0));

    // Check that the HTTP status is OK
    if (req.status == 200) {

      // Store the ResponseText
      var nbaScoreData = req.responseText;

      // Extract an array of games from the ResponseText
      var nbaGames = nbaScoreData.split(/\n/);

      // Sort the game array
      nbaGames = that.SortGameArray(nbaGames);

      // Step through the NBA games for the day
      for (var i = 1; i < nbaGames.length - 1; i++) {

        // Split the game data
        var gameData = nbaGames[i].split("|");
        var gameState = gameData[8].split(";");
        var htm = gameData[11].split(";");
        var vtm = gameData[12].split(";");

        // Extract the code for the current game
        var gameCode = gameData[0];

        // Extract the home team info
        var htmName = that.GetTeam(htm[3], htm[2], htm[4]);
        var htmScore = htm[6];

        // Extract the visitor team info
        var vtmName = that.GetTeam(vtm[3], vtm[2], vtm[4]);
        var vtmScore = vtm[6];

        // Rate the game for determining menu item icon
        var gameRating = that.GetGameRating(gameState[2], htmScore, vtmScore);

        // Create new menu items to add to the popup menus
        var boxscoreItem = document.createElement("menuitem");
        var recapItem = document.createElement("menuitem");

        // Construct the boxscoreItem's label
        var itemLabel = htmName + " " + htmScore + " | " + vtmName + " " + vtmScore;

        // Append overtime alert if applicable
        if (gameState[2] > 4) {
          itemLabel += " (OT)";
        }

        // Set the boxscore menu item's attributes
        boxscoreItem.setAttribute("id", "nba-box-" + scoreboardDate + "/" + gameCode);
        boxscoreItem.setAttribute("label", itemLabel);
        boxscoreItem.setAttribute("oncommand", "ffNBAScoreboard.OpenGameItem(id); event.stopPropagation();");
        boxscoreItem.setAttribute("class", "menuitem-iconic nbascoreboard-rating-" + gameRating);
        boxscoreItem.setAttribute("tooltiptext", "Opens the game boxscore at NBA.com");

        // Set the recap menu item's attributes
        recapItem.setAttribute("id", "nba-rcp-" + scoreboardDate + "/" + gameCode);
        recapItem.setAttribute("label", itemLabel);
        recapItem.setAttribute("oncommand", "ffNBAScoreboard.OpenGameItem(id); event.stopPropagation();");
        recapItem.setAttribute("class", "menuitem-iconic nbascoreboard-rating-" + gameRating);
        recapItem.setAttribute("tooltiptext", "Opens the game recap at NBA.com");

        // Add the items to the menus
        boxscoreMenu.insertBefore(boxscoreItem, boxscoreMenu.firstChild);
        recapMenu.insertBefore(recapItem, recapMenu.firstChild);

      }

      // Insert a blank item if there are no games
      if (boxscoreMenu.childNodes.length === 0) {

        // Create new menu items to add to the popup menu
        var boxBlankItemNoGames = that.CreateBlankMenuItem("boxBlankItem", "There were no games yesterday", "No game information");
        var recapBlankItemNoGames = that.CreateBlankMenuItem("recapBlankItem", "There were no games yesterday", "No game information");

        // Add the items to the menu
        boxscoreMenu.appendChild(boxBlankItemNoGames);
        recapMenu.appendChild(recapBlankItemNoGames);

      }

    // HTTP Request is not OK - boxscores/recaps not available
    } else {

      // Create new menu items to add to the popup menu
      var boxBlankItemNoData = that.CreateBlankMenuItem("boxBlankItem", "Error connecting to data source", "HTTP Error: " + req.status);
      var recapBlankItemNoData = that.CreateBlankMenuItem("recapBlankItem", "Error connecting to data source", "HTTP Error: " + req.status);

      // Add the items to the menu
      boxscoreMenu.appendChild(boxBlankItemNoData);
      recapMenu.appendChild(recapBlankItemNoData);

    }

    // Change to the default cursor
    window.setCursor("auto");

  }

  // Only populate the menus once, when they are empty
  if (boxscoreMenu.childNodes.length === 0) {

    // Add a 'loading' menu item to the boxscore popup menu
    var boxLoadingItem = document.createElement("menuitem");
    boxLoadingItem.setAttribute("id", "boxLoadingItem");
    boxLoadingItem.setAttribute("label", "Connecting to data source...");
    boxLoadingItem.setAttribute("disabled", true);
    boxscoreMenu.appendChild(boxLoadingItem);

    // Add a 'loading' menu item to the recap popup menu
    var recapLoadingItem = document.createElement("menuitem");
    recapLoadingItem.setAttribute("id", "recapLoadingItem");
    recapLoadingItem.setAttribute("label", "Connecting to data source...");
    recapLoadingItem.setAttribute("disabled", true);
    recapMenu.appendChild(recapLoadingItem);

    // Change to the wait cursor
    window.setCursor("progress");

    // Get the date of Today's Scoreboard
    var scoreboardDate = this.GetTodaysDate("si");

    // Change the scoreboardDate to Yesterday's Scoreboard date
    scoreboardDate = this.GetPreviousDate(scoreboardDate, "si");

    // Construct the NBA scores data URL
    var scoreboardURL = "http://sportsillustrated.cnn.com/basketball/nba/scoreboards/" + scoreboardDate + "/scoreboard.dat";

    // Initialise an XMLHttpRequest object for loading HTML
    var req = new XMLHttpRequest();

    // Add event listeners for XMLHttpRequest
    req.addEventListener("progress", updateProgress, false);
    req.addEventListener("load", transferComplete, false);

    // Load the XML data source of the required scoreboard
    req.open("GET", scoreboardURL, true);

    // Initiate the XMLHttpRequest
    req.send(null);

  }

},


// -------------------------------------------------------------------------- //
// Functions called within nbascoreboard.js                                   //
// -------------------------------------------------------------------------- //


/*
 * SortGameArray
 *
 * Returns an array of games sorted chronologically
 * aArray: String array of game data
 *
 */

SortGameArray: function (aArray) {

  // Sort from bottom to top
  for (var j = aArray.length - 2; j > 1; j--) {

    // From top to bottom
    for (var i = 1; i < j; i++) {

      // Get the gameState information to compare
      var str1 = ((aArray[i].split("|"))[8].split(";"))[0].slice(1);
      var str2 = ((aArray[i+1].split("|"))[8].split(";"))[0].slice(1);

      // Switch the arrays for ascending order
      if (str2 > str1) {

        var tempStr = aArray[i+1];
        aArray[i+1] = aArray[i];
        aArray[i] = tempStr;

      }

    }

  }

  return aArray;

},


/*
 * GetTodaysDate
 *
 * Returns the date of Today's Scoreboard
 * - Scoreboard date advances at 12 Noon Eastern (US)
 *
 * aFormat: String 'nba' for YYYYMMDD, 'si' for YYYY/MM/DD
 *
 */

GetTodaysDate: function (aFormat) {

  // Get information from the system clock
  var nowNba = new Date();
  var dayNba = nowNba.getDate();
  var monthNba = nowNba.getMonth() + 1;
  var yearNba = nowNba.getYear() + 1900;
  var regionNba = (nowNba.getTimezoneOffset() / 60) - 5;
  var hourNba = nowNba.getHours();

  // Construct today's date
  var dateToday = this.NumsToDate(dayNba, monthNba, yearNba, aFormat);

  // Get the two previous dates
  var dateYesterday = this.GetPreviousDate(dateToday, aFormat);
  var dateTwoDaysAgo = this.GetPreviousDate(dateYesterday, aFormat);

  // Choose which date to display based on regional settings
  if (hourNba + regionNba < -12) {
    return dateTwoDaysAgo;
  }
  else if (hourNba + regionNba >= 12) {
    return dateToday;
  }
  else {
    return dateYesterday;
  }

},


/*
 * GetPreviousDate
 *
 * Returns the date previous to YYYYMMDD
 * aDate: A date 'YYYYMMDD'
 * aFormat: String 'nba' for YYYYMMDD, 'si' for YYYY/MM/DD
 *
 */

GetPreviousDate: function (aDate, aFormat) {

  // Declare dateString variables
  var yearString = "";
  var monthString = "";
  var dayString = "";

  // Extract information from the NBA.com date string
  if (aFormat == "nba" || aFormat == "espn" || aFormat == "cbs") {
    yearString = aDate.slice(0, 4);
    monthString = aDate.slice(4, 6);
    dayString = aDate.slice(6, 8);
  }

  // Extract information from the SI.com date string
  if (aFormat == "si" || aFormat == "yahoo") {
    yearString = aDate.slice(0, 4);
    monthString = aDate.slice(5, 7);
    dayString = aDate.slice(8, 10);
  }

  // Convert the strings to integers
  var yearInt = parseInt(yearString, 10);
  var monthInt = parseInt(monthString, 10);
  var dayInt = parseInt(dayString, 10);

  // Not the first day of the month
  if (dayInt > 1) {
    dayInt -= 1;
  }

  // The first day of the month
  else {

    // March becomes February
    if (monthInt == 3) {

      // Check for a leap year
      if (yearInt % 4 === 0) {
        dayInt = 29;
      }
      else {
        dayInt = 28;
      }

      // Set month to Feb
      monthInt -= 1;

    }

    // May, July, October, December become April, June, September, November
    else if (monthInt == 10 || monthInt == 5 || monthInt == 7 || monthInt == 12) {
      dayInt = 30;
      monthInt -= 1;
    }

    // January becomes December of the previous year
    else if (monthInt == 1) {
      dayInt = 31;
      monthInt = 12;
      yearInt -= 1;
    }

    // All the other months
    else {
      dayInt = 31;
      monthInt -= 1;
    }

  }

  // Convert the integers to YYYYMMDD
  return this.NumsToDate(dayInt, monthInt, yearInt, aFormat);

},


/*
 * NumsToDate
 *
 * Returns a date from day, month and year
 * aDay, aMonth, aYear: Integers
 * aFormat: String 'nba' for YYYYMMDD, 'si' for YYYY/MM/DD
 */

NumsToDate: function (aDay, aMonth, aYear, aFormat) {

  // Declare a date string
  var dateStr = aYear;

  // Construct the date for NBA.com, ESPN and CBS
  if (aFormat == "nba" || aFormat == "espn" || aFormat == "cbs") {
    dateStr += ((aMonth < 10) ? "0" : "") + aMonth;
    dateStr += ((aDay < 10) ? "0" : "") + aDay;
  }

  // Construct the date for SI.com
  if (aFormat == "si") {
    dateStr += "/" + ((aMonth < 10) ? "0" : "") + aMonth + "/";
    dateStr += ((aDay < 10) ? "0" : "") + aDay;
  }

  // Construct the date for Yahoo!
  if (aFormat == "yahoo") {
    dateStr += "-" + ((aMonth < 10) ? "0" : "") + aMonth + "-";
    dateStr += ((aDay < 10) ? "0" : "") + aDay;
  }

  return dateStr;

},


/*
 * GetGameRating
 *
 */

GetGameRating: function (aPeriod, aHtm, aVtm) {

  // Convert the strings to integers
  var per = parseInt(aPeriod, 10);
  var hScore = parseInt(aHtm, 10);
  var vScore = parseInt(aVtm, 10);

  if (isNaN(hScore) || isNaN(vScore)) {

    // Rating 5 - the game has not started
    return "5";

  } else {

    // Determine the size of the lead
    var lead = Math.abs(hScore - vScore);

    // Determine the lead rating - 1 (small) to 4 (big)
    var leadRating;

    if (lead < 4) {
      leadRating = 1;
    } else if (lead > 3 && lead < 7) {
      leadRating = 2;
    } else if (lead > 6 && lead < 10) {
      leadRating = 3;
    } else {
      leadRating = 4;
    }

    // GameRating 1 - rating 1, 2nd half
    if (leadRating == 1 && per > 2) {
      return "1";
    }

    // GameRating 2 - rating 2, 2nd H, rating 1, 2nd Q
    else if ((leadRating == 2 && per > 2) || (leadRating == 1 && per == 2)) {
      return "2";
    }

    // GameRating 3 - rating 3, 2st H, rating 2, 2nd Q, rating 1, 1st Q
    else if ((leadRating == 3 && per > 2) || (leadRating == 2 && per == 2) || (leadRating == 1 && per == 1)) {
      return "3";
    }

    // GameRating 4 - rating 4, 2nd H, rating 3, 2nd Q, rating 2, 1st Q
    else {
      return "4";
    }

  }

},


/*
 * GetGameStatText
 *
 * Returns game status text
 * aClock: String
 * aPeriod: Integer
 * aActive: Boolean
 * aTime: String
 *
 * Added to translate new SI.com data format to old NBA.com data format
 *
 */

GetGameStatText: function (aClock, aPeriod, aActive, aTime) {

  // Convert the period string to an integer
  var periodInt = parseInt(aPeriod, 10);

  // Determine the game status
  if (periodInt === 0) {
    return this.GetTimeForMenu(aTime);
  }
  else if (periodInt == 1 && aActive == "T") {
    return aClock + " 1st Qtr";
  }
  else if (periodInt == 2 && aActive == "T") {
    return aClock + " 2nd Qtr";
  }
  else if (periodInt == 3 && aActive == "T") {
    return aClock + " 3rd Qtr";
  }
  else if (periodInt == 4 && aActive == "T") {
    return aClock + " 4th Qtr";
  }
  else if (periodInt == 5 && aActive == "T") {
    return aClock + " 1st OT";
  }
  else if (periodInt == 6 && aActive == "T") {
    return aClock + " 2nd OT";
  }
  else if (periodInt == 7 && aActive == "T") {
    return aClock + " 3rd OT";
  }
  else if (periodInt == 1 && (aActive == "F" || aClock == "0:00")) {
    return "End of 1st Qtr";
  }
  else if (periodInt == 2 && (aActive == "F" || aClock == "0:00")) {
    return "Halftime";
  }
  else if (periodInt == 3 && (aActive == "F" || aClock == "0:00")) {
    return "End of 3rd Qtr";
  }
  else if (periodInt >= 4 && (aActive == "F" || aClock == "0:00")) {
    return "Final";
  }
  else {
    return "???";
  }

},


/*
 * GetTeam
 *
 * Returns the name, city or abbreviation of a team
 * aName: A team name, eg Suns
 * aCity: A city name, eg Phoenix
 * aAbrv: An abbreviation, eg PHO
 *
 */

GetTeam: function (aName, aCity, aAbrv) {

  // Retrieve the citynames preference value
  var teamStyle = this.nbaPrefBranch.getCharPref("teamStyle");

  // Return the NBA team name
  if (teamStyle == "teamName") {
    return aName;
  }

  // Return the NBA city name
  else if (teamStyle == "teamCity") {
    return aCity;
  }

  // Return the NBA abbrev name
  else if (teamStyle == "teamAbbrev") {
    return aAbrv;
  }

},


/*
 * GetTimeForMenu
 *
 * Returns the start time of the game, either ET or local time
 * aTime: A time string hh:mm AM/PM
 *
 */

GetTimeForMenu: function (aTime) {

  // Load the time display preference
  var useLocalTime = this.nbaPrefBranch.getBoolPref("useLocalTime");

  if (useLocalTime) {

    // Find today's date as an integer
    var todaysDate = parseInt(this.GetTodaysDate("nba"),10);

    // In 2010, DST ends on November 7 in the US
    var dsEndDate = parseInt("20101107",10);

    // In 2011, DST starts on March 13 in the US
    var dsStartDate = parseInt("20110313",10);

    // Declare a variable for the ET GMT offset
    var etOffset;

    // Set ET GMT offset for EST (-5)
    if (todaysDate >= dsEndDate && todaysDate < dsStartDate) {
      etOffset = -5;
    }

    // Set ET GMT offset for EDT (-4)
    else {
      etOffset = -4;
    }

    // Split up the time string into three parts
    var hours = parseInt(aTime.split(":")[0], 10);
    var mins = (aTime.split(":")[1]).split(" ")[0];
    var ampm = (aTime.split(":")[1]).split(" ")[1];

    // Change 12pm to 0pm
    if (hours == 12 && ampm == "PM") {
      hours = 0;
    }

    // Get region information from the system clock
    var region = ((new Date()).getTimezoneOffset() / 60);

    // Calculate the local time in hours
    localHours = hours - region - etOffset;

    // Adjust for AM/PM
    if (localHours < 0) {
      localHours += 12;
      if (ampm == "AM") {
        ampm = "PM";
      }
      else {
        ampm = "AM";
      }
    }
    else if (localHours === 0) {
      localHours = 12;
    }
    else if (localHours > 11 && localHours < 24) {
      localHours -= 12;
      if (ampm == "AM") {
        ampm = "PM";
      }
      else {
        ampm = "AM";
      }
    }
    else if (localHours == 24) {
      localHours = 12;
    }
    else if (localHours > 24) {
      localHours -= 24;
    }

    // Return the local time
    return localHours + ":" + mins + " " + ampm;

  }

  else {

    // Return Eastern Time
    return aTime + " ET";

  }

},


/*
 * ConfigureMenu
 *
 */

ConfigureMenu: function () {

  // Load the menu preferences
  var showLeaders = this.nbaPrefBranch.getBoolPref("showLeaders");
  var showPrev = this.nbaPrefBranch.getBoolPref("showPrevious");
  var scoreboardType = this.nbaPrefBranch.getCharPref("scoreboard");

  // Load the menu items
  var leaderMenu = document.getElementById("nbascoreboard-leader-menu");
  var yesterMenu = document.getElementById("nbascoreboard-Y-menu");
  var prevMenu = document.getElementById("nbascoreboard-prev-menu");

  // Configure 'Launch Today's Leaderboard'
  leaderMenu.setAttribute("hidden", !showLeaders);

  // Configure 'Previous NBA Scoreboard'
  prevMenu.setAttribute("hidden", !showPrev);

  // Choose icon for yesterday and previous scoreboard menus
  yesterMenu.setAttribute("class", "menuitem-iconic nbascoreboard-" + scoreboardType + "-menu");
  prevMenu.setAttribute("class", "menuitem-iconic nbascoreboard-" + scoreboardType + "-menu");

},


/*
 * PurgeMenu
 *
 */

PurgeMenu: function () {

  // Get the menu elements to be purged
  var tBoxscoreMenu = document.getElementById("nbascoreboard-menu");
  var tRecapMenu = document.getElementById("nbascoreboard-TR-popup");
  var yBoxscoreMenu = document.getElementById("nbascoreboard-YB-popup");
  var yRecapMenu = document.getElementById("nbascoreboard-YR-popup");

  // Purge todays's boxscore menu
  while (tBoxscoreMenu.childNodes.length > this.DEFAULT_MENUITEM_TOTAL) {
    tBoxscoreMenu.removeChild(tBoxscoreMenu.childNodes.item(this.LIVE_SCORES_MENUITEM));
  }

  // Purge today's recap menu
  while (tRecapMenu.childNodes.length > 0) {
    tRecapMenu.removeChild(tRecapMenu.firstChild);
  }

  // Purge yesterday's boxscore menu
  while (yBoxscoreMenu.childNodes.length > 0) {
    yBoxscoreMenu.removeChild(yBoxscoreMenu.firstChild);
  }

  // Purge yesterday's recap menu
  while (yRecapMenu.childNodes.length > 0) {
    yRecapMenu.removeChild(yRecapMenu.firstChild);
  }

  // Hide today's recap menu
  document.getElementById("nbascoreboard-TR-menu").setAttribute("hidden", true);

},


/*
 * CreateBlankMenuItem
 *
 * Returns a blank menu item
 * aID: The text for the item ID
 * aLabel: The text for the label
 * aTip: The text for the tooltip
 *
 */

CreateBlankMenuItem: function (aID, aLabel, aTip) {

  // Create a new menu item to add to the popup menu
  var blankItem = document.createElement("menuitem");

  // Set the new menu item's attributes
  blankItem.setAttribute("id", aID);
  blankItem.setAttribute("oncommand", "event.stopPropagation();");
  blankItem.setAttribute("disabled", true);
  blankItem.setAttribute("label", aLabel);
  blankItem.setAttribute("tooltiptext", aTip);

  // Return a blank item
  return blankItem;

},


// -------------------------------------------------------------------------- //
// Functions called from nba-leaderboard.xul                                  //
// -------------------------------------------------------------------------- //

/*
 * LoadLeaderboard
 *
 */

loadLeaderboard: function () {

  // Get a reference to the ffNBAScoreboard object
  let that = this;

  // Get the title label of the NBA Leaderboard
  var title = document.getElementById("nbaleaderboard-title");

  // Change to the wait cursor
  window.setCursor("progress");

  // Get the date of Today's Scoreboard
  var scoreboardDate = this.GetTodaysDate("si");

  // Construct the NBA scoreboard data URL
  var scoreboardURL = "http://sportsillustrated.cnn.com/basketball/nba/scoreboards/" + scoreboardDate + "/scoreboard.dat";

  // XMLHttpRequest 'progress' event function
  function updateProgress(evt) {

    if (evt.lengthComputable) {
      var percentComplete = parseInt((evt.loaded / evt.total) * 100, 10);
      title.setAttribute("value", "Loading (" + percentComplete + "%)");
    }

  }

  // XMLHttpRequest 'load' event function
  function transferComplete(evt) {

    // Check that the HTTP status is OK
    if (req.status == 200) {

      // Store the ResponseXML in an xmlDoc object
      var nbaScoreData = req.responseText;

      // Extract an array of games from the ResponseText
      var nbaGames = nbaScoreData.split(/\n/);

      // Sort the game array
      nbaGames = that.SortGameArray(nbaGames);

      // Step through the NBA games for the day
      for (var i = 1; i < nbaGames.length - 1; i++) {

        // Determine the gamebox to use (NBA.com --> SI.com merge)
        var boxNo = nbaGames.length - 2 - i;

        // Get a reference to the current game box
        var leaderBox = document.getElementById("nbaleaderboard-gamebox" + boxNo);

        // Split the game data
        var gameData = nbaGames[i].split("|");
        var gameState = gameData[8].split(";");
        var htm = gameData[11].split(";");
        var vtm = gameData[12].split(";");

        // Extract the status of the current game
        var gameStatus = gameData[5];

        // Extract the status description of the current game
        var gameStatText = that.GetGameStatText(gameState[1], gameState[2], gameState[3], gameData[7]);

        // Initialise the game box
        that.initGameBox(leaderBox, htm, vtm);

        // Add the leaders to the game box
        that.addGameLeaders(leaderBox, htm, vtm);

        // Add gameStatText to the game box
        that.addGameStatText(leaderBox, gameStatus, gameStatText);

      }

      // Display a message when there are no games
      if (nbaGames.length < 3) {
        title.setAttribute("value", "There are no games today");
      }

      // Hide the loading label
      else {
        title.setAttribute("hidden", true);
      }

      // Change to the default cursor
      window.setCursor("auto");

      // Resize the window and position it in the centre of the screen
      window.sizeToContent();
      window.screenX = (window.screen.width - window.outerWidth) / 2;
      window.screenY = (window.screen.height - window.outerHeight) / 2;

    }

    // HTTP Request is not OK - leaderboard not available
    else {
      title.setAttribute("value", "Leaderboard not available");
    }

  }

  // Initialise an XMLHttpRequest object for loading XML
  var req = new XMLHttpRequest();

  // Add event listeners for XMLHttpRequest
  req.addEventListener("progress", updateProgress, false);
  req.addEventListener("load", transferComplete, false);

  // Load the XML data source of the required scoreboard
  req.open("GET", scoreboardURL, true);

  // Initiate the XMLHttpRequest
  req.send(null);

},


// -------------------------------------------------------------------------- //
// Functions called within nba-leaderboard.js                                 //
// -------------------------------------------------------------------------- //


/*
 * InitGameBox
 *
 * aGameBox: The XUL game box element
 * aHtm: Text array of information about the home team
 * aVtm: Text array of information about the visitor team
 *
 */

initGameBox: function (aGameBox, aHtm, aVtm, aGameStatus) {

  // Create a new CategoryBox
  var categoryBox = document.createElement("vbox");
  categoryBox.setAttribute("class", "categorybox");

  // Add the category labels
  categoryBox.appendChild(this.createLabel("", false));
  categoryBox.appendChild(this.createLabel("Pts", false));
  categoryBox.appendChild(this.createLabel("Rbd", false));
  categoryBox.appendChild(this.createLabel("Ast", false));

  // Create a new HtmBox
  var htmBox = document.createElement("vbox");

  // Extract the htm labels
  var htmName = this.shortenTeam(this.GetTeam(aHtm[3], aHtm[2], aHtm[4]));

  // Add the htm labels
  htmBox.appendChild(this.createLabel(htmName, true));

  // Create a new VtmBox
  var vtmBox = document.createElement("vbox");

  // Extract the vtm labels
  var vtmName = this.shortenTeam(this.GetTeam(aVtm[3], aVtm[2], aVtm[4]));
  
  // Add the vtm labels
  vtmBox.appendChild(this.createLabel(vtmName, true));

  // Insert the categoryBox into the container
  aGameBox.appendChild(categoryBox);
  aGameBox.appendChild(htmBox);
  aGameBox.appendChild(vtmBox);
  aGameBox.setAttribute("class", "gamebox");

},


/*
 * AddGameLeaders
 *
 * aGameBox: The XUL game box element
 * aHtm: Text array of information about the home team
 * aVtm: Text array of information about the visitor team
 *
 */

addGameLeaders: function (aGameBox, aHtm, aVtm) {

  // Get the htmBox and the htmTitle
  var htmBox = aGameBox.childNodes[1];
  var htmTitle = htmBox.childNodes[0];

  // Get the vtmBox and the vtmTitle
  var vtmBox = aGameBox.childNodes[2];
  var vtmTitle = vtmBox.childNodes[0];

  // Extract the htm labels
  var htmScore = aHtm[6];
  var htmPts = this.getLeader(aHtm[9]);
  var htmRbd = this.getLeader(aHtm[10]);
  var htmAst = this.getLeader(aHtm[11]);

  // Add the htm labels
  htmTitle.setAttribute("value", htmTitle.value + " " + htmScore);
  htmBox.appendChild(this.createLabel(htmPts, false));
  htmBox.appendChild(this.createLabel(htmRbd, false));
  htmBox.appendChild(this.createLabel(htmAst, false));

  // Extract the vtm labels
  var vtmScore = aVtm[6];
  var vtmPts = this.getLeader(aVtm[9]);
  var vtmRbd = this.getLeader(aVtm[10]);
  var vtmAst = this.getLeader(aVtm[11]);

  // Add the vtm labels
  vtmTitle.setAttribute("value", vtmTitle.value + " " + vtmScore);
  vtmBox.appendChild(this.createLabel(vtmPts, false));
  vtmBox.appendChild(this.createLabel(vtmRbd, false));
  vtmBox.appendChild(this.createLabel(vtmAst, false));

},


/*
 * AddGameStatText
 *
 * aGameBox: The XUL game box element
 * aStatus: gameStatus value
 * aText: gameStatText value
 *
 */

addGameStatText: function (aGameBox, aStatus, aText) {

  // Create a new StatBox
  var statBox = document.createElement("vbox");

  // Add the gameStatText
  statBox.appendChild(this.createLabel(aText, false));

  // Colour the statBox
  if (aStatus == "" || aStatus == "PRE" || aStatus == "SCHE") {
    statBox.setAttribute("class", "statbox notstarted");
  } else if (aStatus == "SUSP" || aStatus == "CANC" || aStatus =="FINA") {
    statBox.setAttribute("class", "statbox finished");
  } else {
    statBox.setAttribute("class", "statbox inprogress");
  }

  // Insert the statBox into the container
  var parent = aGameBox.parentNode;
  parent.appendChild(statBox);

},


/*
 * CreateLabel
 *
 * Returns a xUL label element with value and styling
 * aValue: The label's value
 * aTitle: True, title styling. False, default styling
 *
 */

createLabel: function (aValue, aTitle) {

  // Create a new label
  var newLabel = document.createElement("label");

  // Set the value
  newLabel.setAttribute("value", aValue);

  // See the class
  if (aTitle) {
    newLabel.setAttribute("class", "gametitle");
  }

  // Return the new label
  return newLabel;

},


/*
 * GetLeader
 *
 * Returns the leader's name and total
 * aData: A string of leader data
 *
 */

getLeader: function (aData) {

  // Split the leader information
  var playerInfo = aData.split("#");

  // Check for no leader
  if (playerInfo[4] == "0") {
    return "No leader";
  }

  // Display the name and number
  else {
    return playerInfo[1] + " " + playerInfo[4];
  }

},


/*
 * ShortenTeam
 *
 * Returns an abbreviated team name for some teams
 * aTeam: A team name
 *
 */

shortenTeam: function (aTeam) {

  switch (aTeam) {

  case "Timberwolves":
    aTeam = "Wolves";
    break;


  case "Trail Blazers":
    aTeam = "Blazers";
    break;

  case "Mavericks":
    aTeam = "Mavs";
    break;

  default:

  }

  // Return the modified team
  return aTeam;

},


// -------------------------------------------------------------------------- //
// NBA Scoreboard Lite Functions                                              //
// -------------------------------------------------------------------------- //

/*
 * - init: Adds a listener for each page loading
 * - onPageLoad: Executes when a page has loaded
 *
 */

// Extension version
VERSION: "4.11.1",

// Improve start up performance. Code taken from...
//   https://developer.mozilla.org/en/XUL_School/Appendix_A%3A_Add-on_Performance#Startup

init: function() {
  let that = this;
  window.setTimeout(function () { that.postInit(); }, 500);
},


postInit: function () {

  // Get a reference to the ffNBAScoreboard object
  let that = this;

  // Check for an update to the extension
  if (!this.nbaPrefBranch.prefHasUserValue("version") || this.nbaPrefBranch.getCharPref("version") != this.VERSION) {

    // Set the thanksURL
    thanksURL = "http://www.nbascoreboard.net/thank-you/";

    // Open the URL in a tab
    setTimeout(function() { window.openUILinkIn(thanksURL, "tab"); }, 500);

    // Update the version preference
    this.nbaPrefBranch.setCharPref("version", this.VERSION);

  }

  // Retrieve the enableSBLite preference value
  if (this.nbaPrefBranch.getBoolPref("enableSBLite")) {

    // appcontent is the browser target where web pages are loaded
    var appcontent = document.getElementById("appcontent");

    // Add a listener to the appcontent to alert to the page loading
    if (appcontent) {
      appcontent.addEventListener("DOMContentLoaded", that.onPageLoad, true);
    }

  }

},


cleanup: function () {
 
  // appcontent is the browser target where web pages are loaded
  var appcontent = document.getElementById("appcontent");
 
  // Remove the listener to the appcontent to alert to the page loading
  if (appcontent) {
    appcontent.removeEventListener("DOMContentLoaded", ffNBAScoreboard.onPageLoad, true);
  }
 
},


onPageLoad: function(aEvent) {

  // Get a reference to the document
  var doc = aEvent.originalTarget;

  // Check for HTML document
  if (doc instanceof HTMLDocument) {

/* ********************************************************************************************** *
 * NBA.COM SCOREBOARD LITE
 * ********************************************************************************************** */

    // Specify a regular expression to match the NBA.com GameLine URL
    var nbaScoreboardRE = /http:\/\/www\.nba\.com\/gameline\//;

    // Look for the NBA.com Scoreboard
    if (nbaScoreboardRE.test(doc.location.href)) {

      var nbaChrome = doc.getElementById("nbaChrome");
      nbaChrome.parentNode.removeChild(nbaChrome);

      var nbaExpandPod = doc.getElementById("nbaExpandPod");
      nbaExpandPod.parentNode.removeChild(nbaExpandPod);

      var nbaHeaderNav = doc.getElementById("nbaHeaderNav");
      nbaHeaderNav.parentNode.removeChild(nbaHeaderNav);

      var nbaSSHeader = doc.getElementById("nbaSSHeader");
      nbaSSHeader.parentNode.removeChild(nbaSSHeader);

      var nbaGLfooterAd = doc.getElementById("nbaGLfooterAd");
      nbaGLfooterAd.parentNode.removeChild(nbaGLfooterAd);

      var nbaFooter = doc.getElementById("nbaFooter");
      nbaFooter.parentNode.removeChild(nbaFooter);

      // Create new CSS rules to add to the NBA.com Scoreboard
      var headNBA = doc.getElementsByTagName("head")[0];
      var cssNBA = doc.createElement("style");
      cssNBA.setAttribute("type","text/css");
      cssNBA.setAttribute("media","screen");

      // Rule 1: Increase the size of the nbascoreboard.net header
      cssNBA.appendChild(doc.createTextNode("#divHeader { font-size: 14px; font-weight: bold;}\n"));

      // Add new CSS rules to the NBA.com Scoreboard
      headNBA.appendChild(cssNBA);

      // Create the NBAScoreboard.net link for NBA.com Scoreboard
      ffNBAScoreboard.addLinkToPage("nbaContent", doc);

    }

/* ********************************************************************************************** *
 * ESPN SCOREBOARD LITE
 * ********************************************************************************************** */

    // Specify a regular expression to match the ESPN Scoreboard URL
    var espnScoreboardRE = /http:\/\/scores\.espn\.go\.com\/nba\/scoreboard.*/;

    // Look for the ESPN.com Scoreboard
    if (espnScoreboardRE.test(doc.location.href)) {

      var header = doc.getElementById("header");
      header.parentNode.removeChild(header);

      var footer = doc.getElementById("footer");
      footer.parentNode.removeChild(footer);

      var upperad = doc.getElementById("upperad");
      upperad.parentNode.removeChild(upperad);

      var a1x1_ad = doc.getElementById("1x1_ad");
      a1x1_ad.parentNode.removeChild(a1x1_ad);

      var navwrapper = doc.getElementById("nav-wrapper");
      navwrapper.parentNode.removeChild(navwrapper);

      var adsonar = doc.getElementsByTagName("iframe");
      adsonar[0].parentNode.removeChild(adsonar[0]);

      // Retrieve the DIVs
      var divsESPN = doc.getElementsByTagName("div");
      for (i = divsESPN.length - 1; i > 0; i--) {

        if (divsESPN[i].getAttribute("class") == "span-6 ad pencil top") {
          divsESPN[i].parentNode.removeChild(divsESPN[i]);
        }

        if (divsESPN[i].getAttribute("class") == "span-6 ad banner top") {
          divsESPN[i].parentNode.removeChild(divsESPN[i]);
        }

        if (divsESPN[i].getAttribute("class") == "span-6 last") {
          divsESPN[i].parentNode.removeChild(divsESPN[i]);
        }

        if (divsESPN[i].getAttribute("class") == "span-2 last" && divsESPN[i].getAttribute("id") != "gamesRight") {
          divsESPN[i].parentNode.removeChild(divsESPN[i]);
        }

        if (divsESPN[i].getAttribute("class") == "gradientBG") {
          divsESPN[i].parentNode.removeChild(divsESPN[i]);
        }

        if (divsESPN[i].getAttribute("class") == "mod-container mod-no-border") {
          divsESPN[i].parentNode.removeChild(divsESPN[i]);
        }

      }

      // Create new CSS rules to add to the SI.com Scoreboard
      var headESPN = doc.getElementsByTagName("head")[0];
      var cssESPN = doc.createElement("style");
      cssESPN.setAttribute("type","text/css");
      cssESPN.setAttribute("media","screen");

      // Rule 1: Decrease the width of the content <DIV>
      cssESPN.appendChild(doc.createTextNode("#content { width: 614px !important;}\n"));

      // Rule 2: Decrease the padding of the content <DIV>
      cssESPN.appendChild(doc.createTextNode("#content { padding-top: 10px !important;}\n"));

      // Rule 3: Increase the size of the nbascoreboard.net header
      cssESPN.appendChild(doc.createTextNode("#divHeader { font-size: 14px; font-weight: bold;}\n"));

      // Rule 4: Change the colour of the nbascoreboard.net header
      cssESPN.appendChild(doc.createTextNode("#divHeader { background-color: #FFF;}\n"));

      // Add new CSS rules to the ESPN.com Scoreboard
      headESPN.appendChild(cssESPN);

      // Create the NBAScoreboard.net link for ESPN.com Scoreboard
      ffNBAScoreboard.addLinkToPage("content", doc);

    }

/* ********************************************************************************************** *
 * SPORTS ILLUSTRATED SCOREBOARD LITE
 * ********************************************************************************************** */

    // Specify a regular expression to match the SI Scoreboard URL
    var siScoreboardRE = /http:\/\/sportsillustrated\.cnn\.com\/basketball\/nba\/scoreboards\/.*/;

    // Look for the SI.com Scoreboard
    if (siScoreboardRE.test(doc.location.href)) {

      // Retrieve the DIVs
      var divsSI = doc.getElementsByTagName("div");
      for (i = divsSI.length - 1; i > 0; i--) {

        if (divsSI[i].getAttribute("class") == "cnnPersonalize") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnSearch") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnBanner") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnClear") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnTopnav") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnBotnav") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnViewerAd") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnFooterBox") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnStatsLogo") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnSLSpot") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("class") == "cnnSearchFooter") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

        if (divsSI[i].getAttribute("id") == "cnnMlbLftAdSec") {
          divsSI[i].parentNode.removeChild(divsSI[i]);
        }

      }

      // Retrieve the TABLEs
      var tablesSI = doc.getElementsByTagName("table");
      for (i = tablesSI.length - 1; i > 0; i--) {

        if (tablesSI[i].getAttribute("class") == "cnn6pxTmar") {
          tablesSI[i].parentNode.removeChild(tablesSI[i]);
        }

      }

      // Retrieve the TRs
      var trsSI = doc.getElementsByTagName("tr");
      for (i = trsSI.length - 1; i > 0; i--) {

        if (trsSI[i].getAttribute("class") == "cnnWeekNavigation") {
          trsSI[i].parentNode.removeChild(trsSI[i]);
        }

      }

      // Create new CSS rules to add to the SI.com Scoreboard
      var headSI = doc.getElementsByTagName("head")[0];
      var cssSI = doc.createElement("style");
      cssSI.setAttribute("type","text/css");
      cssSI.setAttribute("media","screen");

      // Rule 1: Decrease the width of the cnnContent <DIV>
      cssSI.appendChild(doc.createTextNode(".cnnContent { width: 822px !important;}\n"));

      // Rule 2: Increase the size of the nbascoreboard.net header
      cssSI.appendChild(doc.createTextNode("#divHeader { font-size: 14px; font-weight: bold;}\n"));

      // Add new CSS rules to the SI.com Scoreboard
      headSI.appendChild(cssSI);

      // Create the NBAScoreboard.net link for SI.com Scoreboard
      ffNBAScoreboard.addLinkToPage("scoreTable", doc);

    }

/* ********************************************************************************************** *
 * CBS SPORTS SCOREBOARD LITE
 * ********************************************************************************************** */

    // Specify a regular expression to match the CBS Scoreboard URL
    var cbsScoreboardRE = /http:\/\/www\.cbssports\.com\/nba\/scoreboard.*/;

    // Look for the CBS Scoreboard
    if (cbsScoreboardRE.test(doc.location.href)) {

      var tooltip = doc.getElementById("tooltip");
      tooltip.parentNode.removeChild(tooltip);

      var openBar = doc.getElementById("openBar");
      openBar.parentNode.removeChild(openBar);

      var headerRow = doc.getElementById("headerRow");
      headerRow.parentNode.removeChild(headerRow);

      var fastFactsContainer = doc.getElementById("fastFactsContainer");
      fastFactsContainer.parentNode.removeChild(fastFactsContainer);

      var globalNavMenus = doc.getElementById("globalNavMenus");
      globalNavMenus.parentNode.removeChild(globalNavMenus);

      var globalNavContainer = doc.getElementById("globalNavContainer");
      globalNavContainer.parentNode.removeChild(globalNavContainer);
/*
      var postGlobalNav = doc.getElementById("postGlobalNav");
      postGlobalNav.parentNode.removeChild(postGlobalNav);

      var marketingMessageOne = doc.getElementById("marketingMessageOne");
      marketingMessageOne.parentNode.removeChild(marketingMessageOne);

      var sbWrap = doc.getElementById("sbWrap");
      sbWrap.parentNode.removeChild(sbWrap);

      var sbAudioAlertBox = doc.getElementById("sbAudioAlertBox");
      sbAudioAlertBox.parentNode.removeChild(sbAudioAlertBox);

      var live_upgrade = doc.getElementById("live_upgrade");
      live_upgrade.parentNode.removeChild(live_upgrade);

      var button_bar = doc.getElementById("button-bar");
      button_bar.parentNode.removeChild(button_bar);
*/
      var footerCBS = doc.getElementById("footer");
      footerCBS.parentNode.removeChild(footerCBS);

      // Retrieve the DIVs
      var divsCBS = doc.getElementsByTagName("div");
      for (i = divsCBS.length - 1; i > 0; i--) {

        if (divsCBS[i].getAttribute("class") == "column2") {
          divsCBS[i].parentNode.removeChild(divsCBS[i]);
        }

      }

      // Create new CSS rules to add to the SI.com Scoreboard
      var headCBS = doc.getElementsByTagName("head")[0];
      var cssCBS = doc.createElement("style");
      cssCBS.setAttribute("type","text/css");
      cssCBS.setAttribute("media","screen");

      // Rule 1: Decrease the width of the pageContainer <DIV>
      cssCBS.appendChild(doc.createTextNode("#pageContainer { width: 810px !important;}\n"));

      // Rule 2: Increase the size of the nbascoreboard.net header
      cssCBS.appendChild(doc.createTextNode("#divHeader { font-size: 14px; font-weight: bold;}\n"));

      // Rule 3: Change the colour and width of the nbascoreboard.net header
      cssCBS.appendChild(doc.createTextNode("#divHeader { background-color: #FFF; width: 810px; margin: 0 auto;}\n"));

      // Add new CSS rules to the CBS Scoreboard
      headCBS.appendChild(cssCBS);

      // Create the NBAScoreboard.net link for CBS Scoreboard
      ffNBAScoreboard.addLinkToPage("pageContainer", doc);

    }

/* ********************************************************************************************** *
 * YAHOO! SPORTS SCOREBOARD LITE
 * ********************************************************************************************** */

    // Specify a regular expression to match the Yahoo! Scoreboard URL
    var yahooScoreboardRE = /http:\/\/sports\.yahoo\.com\/nba\/scoreboard.*/;

    // Look for the Yahoo! Scoreboard
    if (yahooScoreboardRE.test(doc.location.href)) {

      var container = doc.getElementById("container");
      container.parentNode.removeChild(container);

      // Retrieve the DIVs
      var divsYAHOO = doc.getElementsByTagName("div");
      for (i = divsYAHOO.length - 1; i > 0; i--) {

        if (divsYAHOO[i].getAttribute("class") == "mast") {
          divsYAHOO[i].parentNode.removeChild(divsYAHOO[i]);
        }

        if (divsYAHOO[i].getAttribute("class") == "footer") {
          divsYAHOO[i].parentNode.removeChild(divsYAHOO[i]);
        }

      }

      // Retrieve the TDs
      var tdsYAHOO = doc.getElementsByTagName("td");
      for (i = tdsYAHOO.length - 1; i > 0; i--) {

        if (tdsYAHOO[i].getAttribute("class") == "ysprtcol1") {
          tdsYAHOO[i].parentNode.removeChild(tdsYAHOO[i]);
        }

      }

      // Create new CSS rules to add to the SI.com Scoreboard
      var headYAHOO = doc.getElementsByTagName("head")[0];
      var cssYAHOO = doc.createElement("style");
      cssYAHOO.setAttribute("type","text/css");
      cssYAHOO.setAttribute("media","screen");

      // Rule 1: Decrease the width of the content <TABLE>
      cssYAHOO.appendChild(doc.createTextNode(".yspcontent { width: 800px !important;}\n"));

      // Rule 2: Increase the size of the nbascoreboard.net header
      cssYAHOO.appendChild(doc.createTextNode("#divHeader { font-size: 14px; font-weight: bold;}\n"));

      // Add new CSS rules to the Yahoo! Scoreboard
      headYAHOO.appendChild(cssYAHOO);

      // Create the NBAScoreboard.net link for Yahoo! Scoreboard
      ffNBAScoreboard.addLinkToPage("ysp-leaguescoreboard", doc);

    }

  }

},


addLinkToPage: function(aElement, aDoc) {

  // Get the element to attach the NBAScoreboard.net link to
  var contentElement = aDoc.getElementById(aElement);

  // Create a div to hold the link
  var divHeader = aDoc.createElement("div");
  divHeader.setAttribute("id","divHeader");
  divHeader.setAttribute("style","text-align:center;");

  // Create the link to NBAScoreboard.net
  var nbaLink = aDoc.createElement("a");
  nbaLink.setAttribute("href","http://www.nbascoreboard.net");
  nbaLink.setAttribute("target","_blank");
  nbaLink.appendChild(aDoc.createTextNode("http://www.nbascoreboard.net/\n"));

  // Add the link to the page
  divHeader.appendChild(aDoc.createTextNode("Thanks for using NBA Scoreboard Lite. "));
  divHeader.appendChild(nbaLink);
  var contentParent = contentElement.parentNode;
  contentParent.insertBefore(divHeader, contentElement);

}

// Converted 23/12/2009 to use the ffNBAScoreboard namespace ---------------- //

};


/*
 * A window listener to initialise NBA Scoreboard Lite
 *
 */

window.addEventListener("load", function() {ffNBAScoreboard.init();}, false);
window.addEventListener("unload", function() {ffNBAScoreboard.cleanup();}, false);

